/*
 * OCSP responder
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 2001
 *
 * Copyright (c) 2001 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include "ocspd_engine.h"

/* External imported variables */
extern OCSPD_CONFIG *ocspd_conf;

/* Functions */

#ifdef HAVE_ENGINE
int ocspd_init_engine ( OCSPD_CONFIG *conf ) {

	char *engine = NULL;
	ENGINE *e = NULL;
	int ret = 1;

	syslog(LOG_INFO, "Initialising HSM [%s]", conf->engine_id );
	ENGINE_load_builtin_engines();
	ERR_load_ENGINE_strings();

#ifdef DEBUG
	if( ocspd_conf->verbose ) {
		for(e = ENGINE_get_first(); e != NULL; e = ENGINE_get_next(e)){
			syslog( LOG_INFO, "Available Engine [ %s ]",
							ENGINE_get_id(e));
		}
	}
#endif

	engine = conf->engine_id;
	if (engine != NULL) {
		if((conf->engine = ENGINE_by_id(engine)) == NULL) {
			syslog(LOG_ERR,"invalid engine \"%s\"", engine);
			return(0);
		}

		syslog(LOG_INFO, "Executing HSM PRE commands" );
		ret = ocspd_exec_engine_cmds( conf->engine, conf->pre_cmds );
		if( ret == 0 ) {
			syslog(LOG_ERR,"error in PRE commands [ %s ]", engine);
			ENGINE_free(e);
			return 0;
		}

		if(!ENGINE_init(conf->engine)) {
			syslog(LOG_ERR,"Error in Engine Init");
         		ENGINE_free(e);
         		return 0;
     		}
		ENGINE_free( conf->engine );

		syslog(LOG_INFO, "Executing HSM POST commands" );
		ret = ocspd_exec_engine_cmds( conf->engine, conf->post_cmds );
		if( ret == 0 ) {
			syslog(LOG_ERR,"error in POST commands [ %s ]", engine);
			return 0;
		}

		if(!ENGINE_set_default(conf->engine, ENGINE_METHOD_ALL)) {
#ifdef DEBUG
			BIO *bio_out = NULL;
#endif

			syslog(LOG_ERR,"ERROR::ENGINE::can't set default "
						" [ %s ]", engine);

#ifdef DEBUG
			if( bio_out=BIO_new_fp(stdout,BIO_NOCLOSE)) {
				ERR_print_errors(bio_out);
			}
#endif
			return(0);
		}
	}

	syslog(LOG_ERR,"Engine Initialisation Complete [\"%s\"]", engine);

	return 1;
}

int ocspd_exec_engine_cmds ( ENGINE *e, STACK *cmds ) {

	int ret = 1;
	int i, val, num;

	/* Check if there are some commands to be executed */
#if OPENSSL_VERSION_NUMBER >= 0x10000000L
        val = sk_OPENSSL_STRING_num(cmds);
#else
        val = sk_num(cmds);
#endif
        if(val < 1) {
		if( ocspd_conf->verbose )
                    syslog(LOG_INFO, "HSM no commands to execute in stack");

                return 1;
        }

	/* Check if the loaded ENGINE has CTRL FUNCTION */
	if(!ENGINE_ctrl(e, ENGINE_CTRL_HAS_CTRL_FUNCTION, 0, NULL, NULL) ||
		((num = ENGINE_ctrl(e, ENGINE_CTRL_GET_FIRST_CMD_TYPE,
						0, NULL, NULL)) <= 0)) {
			if ( ocspd_conf->verbose )  {
				syslog(LOG_INFO, "HSM has no CTRL FUNCTION");
				/*
				if( bio_out=BIO_new_fp(stderr, BIO_NOCLOSE)) {
	                        	ERR_print_errors(bio_out);
					BIO_free ( bio_out );
				}
				*/
			}
		return 1;
	}

	/* Now executes the STACK of commands */
        for(i = 0; i < val; i++) {
                char buf[256];
                const char *cmd = NULL;
		const char *arg = NULL;

#if OPENSSL_VERSION_NUMBER >= 0x10000000L
                if( (cmd = (const char *)sk_OPENSSL_STRING_value(cmds, i)) == NULL ) {
#else
                if( (cmd = sk_value(cmds, i)) == NULL ) {
#endif
			continue;
		}

                /* Check if this command has no ":arg" */
                if((arg = strstr(cmd, ":")) == NULL) {
                        if(!ENGINE_ctrl_cmd_string(e, cmd, NULL, 0)) {
				syslog( LOG_ERR, 
					"ENGINE::ERROR::[%s]", cmd);
                                ret = 0;
			}
		} else {
                        if((int)(arg - cmd) > 254) {
                                syslog(LOG_ERR,"ERROR::ENGINE:: command "
					"name too long [%s] ", cmd );
                                return(0);
			}
			memcpy(buf, cmd, (int)(arg - cmd));
			buf[arg-cmd] = '\0';
			arg++;

                        /* Call the command with the argument */
                        if(!ENGINE_ctrl_cmd_string(e, buf, arg, 0)) {
				if ( ocspd_conf->verbose ) {
					syslog( LOG_ERR, 
						"ERROR::ENGINE::CMD [%s=%s]",
							buf, arg);
				}
                                ret = 0;
			}
                }

		/* Check the return code */
                if(ret == 1) {
			if (ocspd_conf->verbose) {
				syslog(LOG_INFO, "HSM command success [%s]",
					cmd );
			}
		} else {
			syslog(LOG_ERR, "HSM command failure [%s]", cmd );
#ifdef DEBUG
			if ( ocspd_conf->verbose )  {
				if( bio_out=BIO_new_fp(stderr, BIO_NOCLOSE)) {
	                        	ERR_print_errors(bio_out);
					BIO_free ( bio_out );
				}
			}
#endif
                }
	}

	return ( ret );
}

int ocspd_load_engine_section ( OCSPD_CONFIG *conf, char *engine_section ) {

        STACK_OF(CONF_VALUE) *nval = NULL;
        CONF_VALUE *val = NULL;
	int i;

	/* Check for the existance of the dbms_section in config file */
        if (!(nval = NCONF_get_section(conf->conf, engine_section)))
		return 0;

	/* Initialise the STACK for pre/post commands */
#if OPENSSL_VERSION_NUMBER >= 0x10000000L
	conf->pre_cmds = sk_OPENSSL_STRING_new_null();
	conf->post_cmds = sk_OPENSSL_STRING_new_null();
#else
	conf->pre_cmds = sk_new_null();
	conf->post_cmds = sk_new_null();
#endif

	/* Scan each option and push the value to the right cmds stack */
        for (i = 0; i < sk_CONF_VALUE_num(nval); i++) {
                val = sk_CONF_VALUE_value(nval, i);

		if( strstr( val->name, ENV_OCSPD_ENGINE_ID ) != NULL ) {
                        conf->engine_id = strdup(val->value);
			syslog( LOG_INFO, "Using Engine '%s'", val->value );
                } else if( strstr( val->name, ENV_OCSPD_ENGINE_PRE ) != NULL ) {
#if OPENSSL_VERSION_NUMBER >= 0x10000000L
			sk_OPENSSL_STRING_push( conf->pre_cmds, (OPENSSL_STRING)val->value );
#else
			sk_push( conf->pre_cmds, val->value );
#endif
			if( ocspd_conf->verbose )
			    syslog( LOG_INFO, "Added '%s' to PRE COMMANDS",
				val->value );
		} else if ( strstr(val->name, ENV_OCSPD_ENGINE_POST) != NULL) {
#if OPENSSL_VERSION_NUMBER >= 0x10000000L
			sk_OPENSSL_STRING_push( conf->post_cmds, (OPENSSL_STRING)val->value );
#else
			sk_push( conf->post_cmds, val->value );
#endif
			if( ocspd_conf->verbose )
			    syslog( LOG_INFO, "Added '%s' to POST COMMANDS",
				val->value );
		} else {
			syslog( LOG_INFO, "Not Reckognised '%s' option",
				val->name );
		}
	}

	return(1);
}

#endif
